@@ -273,6 +273,20 @@ test_that("as_tibble() checks for `unique` names by default (#278)", {
     fixed = TRUE
   )
 
+  l3 <- list(x = 1, ... = 2)
+  expect_error(
+    as_tibble(l3),
+    error_column_must_not_be_dot_dot(2, repair = TRUE),
+    fixed = TRUE
+  )
+
+  l4 <- list(x = 1, ..1 = 2)
+  expect_error(
+    as_tibble(l4),
+    error_column_must_not_be_dot_dot(2, repair = TRUE),
+    fixed = TRUE
+  )
+
   df <- list(a = 1, b = 2)
   names(df) <- c("", NA)
   df <- new_tibble(df, nrow = 1)
@@ -400,7 +414,7 @@ test_that("as_tibble.table() supports .name_repair", {
   )
   expect_identical(
     names(as_tibble(x, .name_repair = "universal")),
-    c("a..1", "a..2", "n")
+    c("a...1", "a...2", "n")
   )
 
   x <- table("if" = c(1, 1, 1, 2, 2, 2), "when" = c(3, 4, 5, 3, 4, 5))
@@ -426,7 +440,7 @@ test_that("as_tibble.table() supports .name_repair", {
   )
   expect_identical(
     names(as_tibble(x, .name_repair = "universal")),
-    c("m", "n..2", "n..3")
+    c("m", "n...2", "n...3")
   )
 })
 
@@ -478,6 +492,22 @@ test_that("as_tibble() can convert row names", {
   expect_identical(unclass(tbl_df), unclass(df))
 })
 
+test_that("as_tibble() can convert row names for zero-row tibbles", {
+  df <- data.frame(a = 1:3, b = 2:4, row.names = letters[5:7])[0, ]
+
+  expect_identical(
+    as_tibble(df, rownames = NULL),
+    tibble(a = integer(), b = integer())
+  )
+  expect_identical(
+    as_tibble(df, rownames = "id"),
+    tibble(id = character(), a = integer(), b = integer())
+  )
+  tbl_df <- as_tibble(df, rownames = NA)
+  expect_identical(rownames(tbl_df), rownames(df))
+  expect_identical(unclass(tbl_df), unclass(df))
+})
+
 test_that("as_tibble() throws an error when user turns missing row names into column", {
   df <- data.frame(a = 1:3, b = 2:4)
   expect_error(
@@ -485,6 +515,11 @@ test_that("as_tibble() throws an error when user turns missing row names into co
     error_as_tibble_needs_rownames(),
     fixed = TRUE
   )
+  expect_error(
+    as_tibble(df[0, ], rownames = "id"),
+    error_as_tibble_needs_rownames(),
+    fixed = TRUE
+  )
 })
 
 test_that("as_data_frame is an alias of as_tibble", {
